# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.server.display import Display as ServerDisplay
from pywayland.client.display import Display as ClientDisplay

import pytest


def test_display_socket():
    """Test that we can create a socket and have a client connect to it"""
    s = ServerDisplay()

    s.add_socket("wayland-0")

    with pytest.raises(Exception):
        s.add_socket("wayland-0")

    c = ClientDisplay()

    c.connect()

    assert c._ptr

    c.disconnect()
    s.destroy()
