# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from .zwp_tablet_manager_v2 import ZwpTabletManagerV2  # noqa: F401
from .zwp_tablet_pad_group_v2 import ZwpTabletPadGroupV2  # noqa: F401
from .zwp_tablet_pad_ring_v2 import ZwpTabletPadRingV2  # noqa: F401
from .zwp_tablet_pad_strip_v2 import ZwpTabletPadStripV2  # noqa: F401
from .zwp_tablet_pad_v2 import ZwpTabletPadV2  # noqa: F401
from .zwp_tablet_seat_v2 import ZwpTabletSeatV2  # noqa: F401
from .zwp_tablet_tool_v2 import ZwpTabletToolV2  # noqa: F401
from .zwp_tablet_v2 import ZwpTabletV2  # noqa: F401
