# This file has been autogenerated by the pywayland scanner

# Copyright © 2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Global, Interface, Proxy, Resource


class ZwpXwaylandKeyboardGrabV1(Interface):
    """Interface for grabbing the keyboard

    A global interface used for grabbing the keyboard.
    """

    name = "zwp_xwayland_keyboard_grab_v1"
    version = 1


class ZwpXwaylandKeyboardGrabV1Proxy(Proxy):
    interface = ZwpXwaylandKeyboardGrabV1

    @ZwpXwaylandKeyboardGrabV1.request()
    def destroy(self):
        """Destroy the grabbed keyboard object

        Destroy the grabbed keyboard object. If applicable, the compositor will
        ungrab the keyboard.
        """
        self._marshal(0)
        self._destroy()


class ZwpXwaylandKeyboardGrabV1Resource(Resource):
    interface = ZwpXwaylandKeyboardGrabV1


class ZwpXwaylandKeyboardGrabV1Global(Global):
    interface = ZwpXwaylandKeyboardGrabV1


ZwpXwaylandKeyboardGrabV1._gen_c()
ZwpXwaylandKeyboardGrabV1.proxy_class = ZwpXwaylandKeyboardGrabV1Proxy
ZwpXwaylandKeyboardGrabV1.resource_class = ZwpXwaylandKeyboardGrabV1Resource
ZwpXwaylandKeyboardGrabV1.global_class = ZwpXwaylandKeyboardGrabV1Global
