# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from .wl_buffer import WlBuffer  # noqa: F401
from .wl_callback import WlCallback  # noqa: F401
from .wl_compositor import WlCompositor  # noqa: F401
from .wl_data_device import WlDataDevice  # noqa: F401
from .wl_data_device_manager import WlDataDeviceManager  # noqa: F401
from .wl_data_offer import WlDataOffer  # noqa: F401
from .wl_data_source import WlDataSource  # noqa: F401
from .wl_display import WlDisplay  # noqa: F401
from .wl_keyboard import WlKeyboard  # noqa: F401
from .wl_output import WlOutput  # noqa: F401
from .wl_pointer import WlPointer  # noqa: F401
from .wl_region import WlRegion  # noqa: F401
from .wl_registry import WlRegistry  # noqa: F401
from .wl_seat import WlSeat  # noqa: F401
from .wl_shell import WlShell  # noqa: F401
from .wl_shell_surface import WlShellSurface  # noqa: F401
from .wl_shm import WlShm  # noqa: F401
from .wl_shm_pool import WlShmPool  # noqa: F401
from .wl_subcompositor import WlSubcompositor  # noqa: F401
from .wl_subsurface import WlSubsurface  # noqa: F401
from .wl_surface import WlSurface  # noqa: F401
from .wl_touch import WlTouch  # noqa: F401
